import click
import cnvrg.helpers.cli_helper as cli_helper
from cnvrg.modules.library import Library, DEFAULT_WORKING_DIR
from cnvrg.helpers.logger_helper import log_message
import cnvrg.helpers.export_library_helper as export_library_helper
import os


@click.group()
def library():
    pass

@library.group("library")
def lib_cmd():
    pass

@lib_cmd.command()
@click.argument("library_title")
def info(library_title):
    l = Library(library_title)
    log_message(l)


@lib_cmd.command()
def list():
    [log_message(x) for x in Library.list()]


@lib_cmd.command()
@click.argument("library_title")
@click.option('--working_dir', '-w', default=None, help='working dir')
def clone(library_title, working_dir):
    l = Library(library_title, working_dir=working_dir)
    library_path = l.load()
    log_message("You can find the library in {path}".format(path=library_path), fg='green')



@lib_cmd.command(help="Title")
@click.argument("library_title")
@click.option("--local/--remote", default=False, help="run on your local/remote machine")
@click.option("--arg", multiple=True, help="Add arguments to the cmd in key:value format")
@click.option("--dataset", multiple=True, help="Attach datasets (on remote)")
@click.option("--compute", multiple=True, help="Template to run (on remote)")
@click.option("--working_dir", help="directory to run from", default=None)
@click.option("--title", help="specify experiment title")
def run(library_title, local, arg, dataset, working_dir, compute, title):
    arguments = cli_helper.parse_args(arg)
    datasets = cli_helper.parse_datasets(dataset)
    l = Library(library_title, working_dir=working_dir)
    if local: l.load()
    cli_helper.print_object(l.run(datasets=datasets, computes=compute, title=title, local=local, args=arguments))


@lib_cmd.command(help="Export")
@click.argument("library_files", nargs=-1)
def export(library_files):
    if len(library_files) == 0: library_files = ["."]
    l = Library.export(*library_files)
    log_message(l)


# @lib_cmd()
# @click.option("--working_dir", help="directory to run from", default=None)
# def export(working_dir):
#     export_library_helper.export_library(working_dir=working_dir)


from cnvrg.modules.dataset import Dataset
from cnvrg.modules.errors import UserError
import os
try:
    from torch.utils import data



    class DataLoader(data.DataLoader):
        def __init__(self, dataset, working_dir=None, **kwargs):
            working_dir = working_dir or os.path.expanduser("/data//{dataset_slug}".format(dataset_slug=dataset))
            # Arguments:
            #         dataset (Dataset): dataset from which to load the data.
            #         batch_size (int, optional): how many samples per batch to load
            #             (default: ``1``).
            #         shuffle (bool, optional): set to ``True`` to have the data reshuffled
            #             at every epoch (default: ``False``).
            #         sampler (Sampler, optional): defines the strategy to draw samples from
            #             the dataset. If specified, ``shuffle`` must be False.
            #         batch_sampler (Sampler, optional): like sampler, but returns a batch of
            #             indices at a time. Mutually exclusive with :attr:`batch_size`,
            #             :attr:`shuffle`, :attr:`sampler`, and :attr:`drop_last`.
            #         num_workers (int, optional): how many subprocesses to use for data
            #             loading. 0 means that the data will be loaded in the main process.
            #             (default: ``0``)
            #         collate_fn (callable, optional): merges a list of samples to form a mini-batch.
            #         pin_memory (bool, optional): If ``True``, the data loader will copy tensors
            #             into CUDA pinned memory before returning them.  If your data elements
            #             are a custom type, or your ``collate_fn`` returns a batch that is a custom type
            #             see the example below.
            #         drop_last (bool, optional): set to ``True`` to drop the last incomplete batch,
            #             if the dataset size is not divisible by the batch size. If ``False`` and
            #             the size of dataset is not divisible by the batch size, then the last batch
            #             will be smaller. (default: ``False``)
            #         timeout (numeric, optional): if positive, the timeout value for collecting a batch
            #             from workers. Should always be non-negative. (default: ``0``)
            #         worker_init_fn (callable, optional): If not ``None``, this will be called on each
            #             worker subprocess with the worker id (an int in ``[0, num_workers - 1]``) as
            #             input, after seeding and before data loading. (default: ``None``)
            super(DataLoader, self).__init__(Dataset(dataset, working_dir=working_dir).pytorch_dataset(), **kwargs)

except ImportError:
    class DataLoader():
        def __init__(self):
            raise UserError("Cant import Data Loader pytorch is required.")


from cnvrg.modules.base_module import CnvrgBase
from cnvrg.helpers.auth_helper import CnvrgCredentials
from cnvrg.helpers.apis_helper import update_credentials, credentials
class Cnvrg(CnvrgBase):
    def __init__(self, url="https://app.cnvrg.io", email=None, password=None):
        if email and password:
            self.credentials = credentials
            return
        self.credentials = CnvrgCredentials().login(email, password, api_url=url)
        update_credentials()

    def me(self):
        return self.credentials.username


from .base_files_connector import BaseFileConnector
from cnvrg.modules.dataset import Dataset
from cnvrg.helpers.apis_helper import url_join
DATASET_TYPE_QUERY = "type_query"
DATASET_TYPE_DATASET = "type_dataset"
class DatasetConnector(BaseFileConnector):

    @staticmethod
    def key_type():
        return "dataset"

    def __init__(self, dataset=None, working_dir=None, query=None):
        super(DatasetConnector, self).__init__(dataset)
        self.__type = DATASET_TYPE_DATASET
        self.__query = query
        self.__files = None
        self.__dataset = Dataset(url_join(self._org, self._data_connector), working_dir=working_dir)

    @property
    def _dataset_type(self):
        return DATASET_TYPE_QUERY if self.__query else DATASET_TYPE_DATASET


    @property
    def working_dir(self):
        return self.__dataset.get_working_dir()

    def __len__(self):
        if not self.__files: self.__fetch()
        return len(self.__files)

    def __fetch(self):
        if self._dataset_type == DATASET_TYPE_DATASET:
            self.__files = self._files_callback(self.__dataset.fetch_all_files())
        elif self._dataset_type == DATASET_TYPE_QUERY:
            self.__files = self._files_callback(self.__dataset.fetch_query(self.__query))

    def __getitem__(self, item):
        return self.__dataset.download_file(self.__files[item])